==> hermes-5.7.4.2.tar.gz: hermes-5.7.4.2/spec/apis/dmo/public_housing_offers_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

MUTATE_PARAMS = {
  entry_date: 'DateTime',
  external_ref: 'String',
  communicate_exact_address: 'Boolean',
  typology: 'String',
  lessor_status: 'String',
  type: 'String',
  age: 'String',
  district: 'String',
  publication_end_date: 'DateTime',
  rent_amount: 'Float',
  surface: 'Integer',
  rooms: 'Integer',
  exclusivity_end_date: 'DateTime',
  nb_of_eligible_employees: 'Integer',
  nb_of_requests_from_employees: 'Integer',
  availability_date: 'DateTime',
  description: 'String',
  details: 'String'
}.freeze

ACCEPTABLE_PARAMS = %w[
  id
  created_at
  updated_at
  external_ref
  communicate_exact_address
  typology
  type
  age
  district
  department
  publication_end_date
  rent_amount
  rent_with_charges
  surface
  rooms
  exclusivity_end_date
  availability_date
  description
  details
  effort_rate
  designation_date_end
  min_effort_treshold
  single_parent
  reserved
  already_rent
  reservation_agreement_ref
  priority_public
  mobility
  bedrooms
  bathrooms
  heating_type
  hot_water_distribution
  floor
  floors
  elevators
  has_elevator
  has_gardian
  outside
  cellars
  has_cellar
  parking_type
  pmr
  kitchen_type
  address
  postal_code
  city
  country
  latitude
  longitude
  address_complements
  mutation
  individual_heating
  date_publication_start
  date_publication_end
  kind
  building_type
  guarantee_deposit
  rental_charges
  parking_charges
  offer_status
  offer_status_id
  offer_status_updated_at
  lessor_status
  lessor_status_id
  lessor_status_updated_at
  min_revenues
  total_charges
  min_effort_rate
  tense_area
  subsidiary_company
  rpls_id
  operation_name
  administrator
  reserving_business_name
  administration_area
  potential_request_name_town
  potential_request_name_county
  number_of_companies_30_minutes_around
  construction_type
  construction_year
  address_can_be_shared
  qpv
  total_floors
  address_extended
  test_created_at
  already_inhabited
  new_offer
  lessor_name
  lessor_siren
  als_siren
  first_quartile
  postal_code
  residence_code
  residence_title
  pictures
  pictures_nb
  agency_id
  agency_name
  lessor_contact_name
  lessor_email
  lessor_phone_number
  gardens
  has_garden
  green_area
  terraces_or_balconies
  dpe_ges
  dpe_conso
  condo
  furnished_lodging
  virtual_visit
  duplex
  first_quartile
  other_quartiles
  lessor_convention_reference
  disability_accesses
  has_disability_access
  effort_rate
  lessor_id
  lessor
  entities_siret
  entity_ids
  agency_id
  reference
  release_date
  commercialized
  is_public
  is_mad
  is_mad_updated_at
  mad_als_date
  ordered_picture_ids
  main_picture
  pictures
  reference_mad
  lessor_orientations
].freeze

VALID_OPERATORS = %w[
  $in
  $nin
  $ne
  $gt
  $gte
  $lt
  $lte
  $eq
  $regex
  $all
].freeze

describe Hermes::Api::Dmo::PublicHousingOffersController, type: :api do
  routes { Hermes::Engine.routes }
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper

  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:user) { create(:user) }
  let!(:lessor) { create(:lessor) }


  describe 'POST /public_housing_offers' do
    describe 'Create housing_offer with minimal valid datas' do
      before :each do
        post(
          'create',
          data: {
            attributes: {}
          }
        )
      end
      it 'returns status code 422' do
        expect(response).to have_http_status(422)
      end
    end
    MUTATE_PARAMS.each do |param, type|
      describe "post request with #{param} (#{type}) is valid" do
        let(:params) do
          attr = {}
          attr[param] = get_typed_value(type)
          attr
        end
        before :each do
          post(
            'create',
            data: {
              attributes: params
            }
          )
        end
        it 'returns status code 422' do
          expect(response).to have_http_status(422)
        end
      end
    end
  end

  describe 'GET /public_housing_offers as a guest' do
    describe 'GET /housing_offers' do
      [50, 150, 100].each_with_index do |qtt, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            terraces_or_balconies: qtt,
            offer_status: 'LOGEMENT PUBLIE',
            lessor_id: lessor.id,
          )
        end
      end

      before :each do
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end

    describe 'GET /housing_offers filtered' do
      ['dc', 'dc', 'priority_public'].each_with_index do |channel_key, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            housing_channel_keys: [channel_key],
            offer_status: 'LOGEMENT PUBLIE'
          )
        end
      end
      let!(:entity) {create(:entity)}
      let!(:entity_reservation) {create(:entity_reservation, housing_offer_id: housing_offer1.id, entity_id: entity.id, from: DateTime.parse("01/01/2021"), to: DateTime.parse("01/02/2021"))}

      before :each do
        housing_offer1.active_reservations.push(entity_reservation.id.to_s);
        housing_offer1.save
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end

    describe 'GET /housing_offers filtered' do
      ['dc', 'dc', 'priority_public'].each_with_index do |channel_key, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            housing_channel_keys: [channel_key],
            offer_status: 'LOGEMENT PUBLIE'
          )
        end
      end
      let!(:entity) {create(:entity)}
      let!(:entity_reservation) {create(:entity_reservation, housing_offer_id: housing_offer1.id, entity_id: entity.id, from: DateTime.parse("01/01/2021"), to: DateTime.parse("01/02/2021"))}

      before :each do
        housing_offer1.active_reservations.push(entity_reservation.id.to_s);
        housing_offer1.save
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end

    MUTATE_PARAMS.each do |acceptable_param, type|
      next unless type

      describe "GET /public_housing_offers with sort on field #{acceptable_param} works " do
        [3, 1, 2].each do |i|
          let!("housing_offer#{i}") do
            create(
              :housing_offer,
              offer_status: 'LOGEMENT PUBLIE',
              acceptable_param => get_typed_value(type, i - 1)
            )
          end
        end

        before :each do
          [3, 1, 2].each do |i|
            expect(send("housing_offer#{i}").send(acceptable_param)).to eq get_typed_value(type, i - 1)
          end
          get 'index', sort: { acceptable_param => 1 }
        end

        it 'returns sorted results' do
          expect(response.status).to eq(200)
          data = json['data']
          expect(data.length).to be 3
          housing_offers_attributes = data.map do |unit|
            param = unit['attributes'][acceptable_param.to_s]
            case type
            when 'DateTime'
              DateTime.parse(param)
            when 'Date'
              Date.parse(param)
            else
              param
            end
          end
          expected_result = (0..2).map { |i| get_typed_value(type, i) }
          expect(housing_offers_attributes).to eq(expected_result)
        end
      end
    end

    describe 'GET /public_housing_offers with operators' do
      [
        ['accueillant', 50],
        ['bel appart', 150],
        ['Super', 150]
      ].each_with_index do |attributes, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            offer_status: 'LOGEMENT PUBLIE',
            description: attributes[0],
            terraces_or_balconies: attributes[1]
          )
        end
      end

      let!(:housing_offer4) do
        create(
          :housing_offer,
          description: 'bel appart',
          terraces_or_balconies: 150,
          offer_status: 'NON PUBLIE'
        )
      end

      {
        1 => [
          { terraces_or_balconies: { '$lt': 100 } },
          { description: { '$in': %w[accueillant flamand] } },
          { description: { '$regex': /^S/ } }
        ],
        2 => [
          terraces_or_balconies: { '$ne': 50 }
        ]
      }.each do |qtt, parameters_setup|
        parameters_setup.each do |parameters|
          describe "with #{parameters}" do
            before :each do
              authenticated_header(request, responsible)
              get 'index', parameters
            end
            it 'returns valid offers quantity' do
              expect(response.status).to eq(200)
              expect(json['meta']['pagination']['total_objects']).to eq(qtt)
            end
          end
        end
      end
    end

    describe 'GET /public_housing_offers with dates range' do
      let(:end_dates) { [Date.new(2020, 12, 1), Date.new(2020, 8, 1), Date.new(2020, 7, 1)]}
      3.times do |i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            offer_status: "LOGEMENT PUBLIE",
            date_publication_start: Date.new(2020, 1, 2)
          )
        end
      end
      let!(:housing_offer3) do
        create(
          :housing_offer,
          offer_status: "NON PUBLIE",
          date_publication_start: Date.new(2020, 1, 2),
          publication_end_date: Date.new(2020, 8, 1)
        )
      end

      before do
        end_dates.each_with_index do |date, i|
          offer = send("housing_offer#{i}")
          offer.set(publication_end_date: date)
        end
        authenticated_header(request, responsible)
        get 'index', { date_publication_start: { '$gte': '2020-01-01' }, publication_end_date: { '$lte': '2020-08-01' } }
      end

      it 'returns valid offers quantity' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq 2
      end
    end
  end

  describe 'GET /public_housing_offers/:id as a guest' do
    describe 'verif it work' do
      let!(:housing_offer1) { create(:housing_offer, description: 'accueillant', terraces_or_balconies: 50, offer_status: 'LOGEMENT PUBLIE') }
      before :each do
        get 'show', id: housing_offer1.id
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end
  end

  describe 'DELETE /public_housing_offers/:id as a guest' do
    let!(:housing_offer) { create(:housing_offer, user_id: responsible.id.to_s) }
    before :each do
      delete 'destroy',
              id: housing_offer.id.to_s
    end
    it 'returns status code 422' do
      expect(response).to have_http_status(422)
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::PublicHousingOffersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  TAGS_API_DMO_PUBLIC_HOUSING_OFFERS = 'API_DMO_PUBLIC_HOUSING_OFFERS'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) { create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator']) }
  let!(:user) { create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com') }
  let!(:applicant) { create(:applicant, user: responsible) }
  let!(:housing_request) { create(:housing_request, main_applicant_id: applicant.id.to_s) }

  context 'as guest' do
    let!(:admin) { create(:administrator) }

    path '/api/dmo/public_housing_offers' do
      get 'Retrieves all public housing_offers' do
        tags TAGS_API_DMO_PUBLIC_HOUSING_OFFERS
        produces 'application/json'
        parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
        ACCEPTABLE_PARAMS.each do |acceptable_param|
          type = :string
          if MUTATE_PARAMS[acceptable_param]
            type = nice_type(MUTATE_PARAMS[acceptable_param])
          end
          parameter name: acceptable_param.to_s, in: :query, type: type, required: false
        end
        # VALID_OPERATORS.each do |valid_operator|
        #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
        # end
        paginator

        response '200', 'housing_offers found' do
          let!(:housing_offers) { create_list(:housing_offer, 5, offer_status: 'LOGEMENT PUBLIE') }

          include_context 'with integration test'
          include_context 'valid jsonapi collection' do
            let(:expected_collection) { housing_offers }
          end
        end
      end
    end

    path '/api/dmo/public_housing_offers/{id}' do
      get 'Retrieves a single housing_offer by id' do
        tags TAGS_API_DMO_PUBLIC_HOUSING_OFFERS
        produces 'application/json'
        parameter name: :id, in: :path, type: :string, required: true
        response '200', 'housing_offer found' do
          let(:housing_offer) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE') }
          let(:id) { housing_offer.id }
          include_context 'with integration test'
          include_context 'valid jsonapi ressource' do
            let(:expected_ressource) { housing_offer }
          end
        end
      end
    end
  end
end

==> hermes-5.7.5.0.tar.gz: hermes-5.7.5.0/spec/apis/dmo/public_housing_offers_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

MUTATE_PARAMS = {
  entry_date: 'DateTime',
  external_ref: 'String',
  communicate_exact_address: 'Boolean',
  typology: 'String',
  lessor_status: 'String',
  type: 'String',
  age: 'String',
  district: 'String',
  publication_end_date: 'DateTime',
  rent_amount: 'Float',
  surface: 'Integer',
  rooms: 'Integer',
  exclusivity_end_date: 'DateTime',
  availability_date: 'DateTime',
  description: 'String',
  details: 'String'
}.freeze

ACCEPTABLE_PARAMS = %w[
  id
  created_at
  updated_at
  external_ref
  communicate_exact_address
  typology
  type
  age
  district
  department
  publication_end_date
  rent_amount
  rent_with_charges
  surface
  rooms
  exclusivity_end_date
  availability_date
  description
  details
  effort_rate
  designation_date_end
  single_parent
  reserved
  already_rent
  reservation_agreement_ref
  priority_public
  mobility
  bedrooms
  bathrooms
  heating_type
  hot_water_distribution
  floor
  floors
  elevators
  has_elevator
  has_gardian
  outside
  cellars
  has_cellar
  parking_type
  pmr
  kitchen_type
  address
  postal_code
  city
  country
  latitude
  longitude
  address_complements
  mutation
  individual_heating
  date_publication_start
  date_publication_end
  kind
  building_type
  guarantee_deposit
  rental_charges
  parking_charges
  offer_status
  offer_status_id
  offer_status_updated_at
  lessor_status
  lessor_status_id
  lessor_status_updated_at
  total_charges
  min_effort_rate
  tense_area
  subsidiary_company
  rpls_id
  potential_request_name_town
  potential_request_name_county
  number_of_companies_30_minutes_around
  construction_type
  construction_year
  address_can_be_shared
  qpv
  total_floors
  address_extended
  already_inhabited
  new_offer
  first_quartile
  postal_code
  residence_code
  residence_title
  pictures
  pictures_nb
  gardens
  has_garden
  green_area
  terraces_or_balconies
  dpe_ges
  dpe_conso
  condo
  furnished_lodging
  virtual_visit
  duplex
  first_quartile
  other_quartiles
  lessor_convention_reference
  disability_accesses
  has_disability_access
  effort_rate
  agency_id
  reference
  release_date
  is_public
  ordered_picture_ids
  main_picture
  pictures
].freeze

VALID_OPERATORS = %w[
  $in
  $nin
  $ne
  $gt
  $gte
  $lt
  $lte
  $eq
  $regex
  $all
].freeze

describe Hermes::Api::Dmo::PublicHousingOffersController, type: :api do
  routes { Hermes::Engine.routes }
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper

  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:user) { create(:user) }
  let!(:lessor) { create(:lessor) }


  describe 'POST /public_housing_offers' do
    describe 'Create housing_offer with minimal valid datas' do
      before :each do
        post(
          'create',
          data: {
            attributes: {}
          }
        )
      end
      it 'returns status code 422' do
        expect(response).to have_http_status(422)
      end
    end
    MUTATE_PARAMS.each do |param, type|
      describe "post request with #{param} (#{type}) is valid" do
        let(:params) do
          attr = {}
          attr[param] = get_typed_value(type)
          attr
        end
        before :each do
          post(
            'create',
            data: {
              attributes: params
            }
          )
        end
        it 'returns status code 422' do
          expect(response).to have_http_status(422)
        end
      end
    end
  end

  describe 'GET /public_housing_offers as a guest' do
    describe 'GET /housing_offers' do
      [50, 150, 100].each_with_index do |qtt, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            terraces_or_balconies: qtt,
            offer_status: 'LOGEMENT PUBLIE',
            lessor_id: lessor.id,
          )
        end
      end

      before :each do
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end

    describe 'GET /housing_offers filtered' do
      ['dc', 'dc', 'priority_public'].each_with_index do |channel_key, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            housing_channel_keys: [channel_key],
            offer_status: 'LOGEMENT PUBLIE'
          )
        end
      end
      let!(:entity) {create(:entity)}
      let!(:entity_reservation) {create(:entity_reservation, housing_offer_id: housing_offer1.id, entity_id: entity.id, from: DateTime.parse("01/01/2021"), to: DateTime.parse("01/02/2021"))}

      before :each do
        housing_offer1.active_reservations.push(entity_reservation.id.to_s);
        housing_offer1.save
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end

    describe 'GET /housing_offers filtered' do
      ['dc', 'dc', 'priority_public'].each_with_index do |channel_key, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            housing_channel_keys: [channel_key],
            offer_status: 'LOGEMENT PUBLIE'
          )
        end
      end
      let!(:entity) {create(:entity)}
      let!(:entity_reservation) {create(:entity_reservation, housing_offer_id: housing_offer1.id, entity_id: entity.id, from: DateTime.parse("01/01/2021"), to: DateTime.parse("01/02/2021"))}

      before :each do
        housing_offer1.active_reservations.push(entity_reservation.id.to_s);
        housing_offer1.save
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(1)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end

    MUTATE_PARAMS.each do |acceptable_param, type|
      next unless type

      describe "GET /public_housing_offers with sort on field #{acceptable_param} works " do
        [3, 1, 2].each do |i|
          let!("housing_offer#{i}") do
            create(
              :housing_offer,
              offer_status: 'LOGEMENT PUBLIE',
              acceptable_param => get_typed_value(type, i - 1)
            )
          end
        end

        before :each do
          [3, 1, 2].each do |i|
            expect(send("housing_offer#{i}").send(acceptable_param)).to eq get_typed_value(type, i - 1)
          end
          get 'index', sort: { acceptable_param => 1 }
        end

        it 'returns sorted results' do
          expect(response.status).to eq(200)
          data = json['data']
          expect(data.length).to be 3
          housing_offers_attributes = data.map do |unit|
            param = unit['attributes'][acceptable_param.to_s]
            case type
            when 'DateTime'
              DateTime.parse(param)
            when 'Date'
              Date.parse(param)
            else
              param
            end
          end
          expected_result = (0..2).map { |i| get_typed_value(type, i) }
          expect(housing_offers_attributes).to eq(expected_result)
        end
      end
    end

    describe 'GET /public_housing_offers with operators' do
      [
        ['accueillant', 50],
        ['bel appart', 150],
        ['Super', 150]
      ].each_with_index do |attributes, i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            offer_status: 'LOGEMENT PUBLIE',
            description: attributes[0],
            terraces_or_balconies: attributes[1]
          )
        end
      end

      let!(:housing_offer4) do
        create(
          :housing_offer,
          description: 'bel appart',
          terraces_or_balconies: 150,
          offer_status: 'NON PUBLIE'
        )
      end

      {
        1 => [
          { terraces_or_balconies: { '$lt': 100 } },
          { description: { '$in': %w[accueillant flamand] } },
          { description: { '$regex': /^S/ } }
        ],
        2 => [
          terraces_or_balconies: { '$ne': 50 }
        ]
      }.each do |qtt, parameters_setup|
        parameters_setup.each do |parameters|
          describe "with #{parameters}" do
            before :each do
              authenticated_header(request, responsible)
              get 'index', parameters
            end
            it 'returns valid offers quantity' do
              expect(response.status).to eq(200)
              expect(json['meta']['pagination']['total_objects']).to eq(qtt)
            end
          end
        end
      end
    end

    describe 'GET /public_housing_offers with dates range' do
      let(:end_dates) { [Date.new(2020, 12, 1), Date.new(2020, 8, 1), Date.new(2020, 7, 1)]}
      3.times do |i|
        let!(:"housing_offer#{i}") do
          create(
            :housing_offer,
            offer_status: "LOGEMENT PUBLIE",
            date_publication_start: Date.new(2020, 1, 2)
          )
        end
      end
      let!(:housing_offer3) do
        create(
          :housing_offer,
          offer_status: "NON PUBLIE",
          date_publication_start: Date.new(2020, 1, 2),
          publication_end_date: Date.new(2020, 8, 1)
        )
      end

      before do
        end_dates.each_with_index do |date, i|
          offer = send("housing_offer#{i}")
          offer.set(publication_end_date: date)
        end
        authenticated_header(request, responsible)
        get 'index', { date_publication_start: { '$gte': '2020-01-01' }, publication_end_date: { '$lte': '2020-08-01' } }
      end

      it 'returns valid offers quantity' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq 2
      end
    end
  end

  describe 'GET /public_housing_offers/:id as a guest' do
    describe 'verif it work' do
      let!(:housing_offer1) { create(:housing_offer, user: user, description: 'accueillant', terraces_or_balconies: 50, offer_status: 'LOGEMENT PUBLIE', als_siren: '123456789') }
      before :each do
        get 'show', id: housing_offer1.id
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data']['attributes']['als_siren']).to be_nil
        expect(json['data']['attributes']['responsible']).to eq("")
      end
    end
  end

  describe 'DELETE /public_housing_offers/:id as a guest' do
    let!(:housing_offer) { create(:housing_offer, user_id: responsible.id.to_s) }
    before :each do
      delete 'destroy',
              id: housing_offer.id.to_s
    end
    it 'returns status code 422' do
      expect(response).to have_http_status(422)
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::PublicHousingOffersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  TAGS_API_DMO_PUBLIC_HOUSING_OFFERS = 'API_DMO_PUBLIC_HOUSING_OFFERS'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) { create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator']) }
  let!(:user) { create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com') }
  let!(:applicant) { create(:applicant, user: responsible) }
  let!(:housing_request) { create(:housing_request, main_applicant_id: applicant.id.to_s) }

  context 'as guest' do
    let!(:admin) { create(:administrator) }

    path '/api/dmo/public_housing_offers' do
      get 'Retrieves all public housing_offers' do
        tags TAGS_API_DMO_PUBLIC_HOUSING_OFFERS
        produces 'application/json'
        parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
        ACCEPTABLE_PARAMS.each do |acceptable_param|
          type = :string
          if MUTATE_PARAMS[acceptable_param]
            type = nice_type(MUTATE_PARAMS[acceptable_param])
          end
          parameter name: acceptable_param.to_s, in: :query, type: type, required: false
        end
        # VALID_OPERATORS.each do |valid_operator|
        #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
        # end
        paginator

        response '200', 'housing_offers found' do
          let!(:housing_offers) { create_list(:housing_offer, 5, offer_status: 'LOGEMENT PUBLIE') }

          include_context 'with integration test'
          include_context 'valid jsonapi collection' do
            let(:expected_collection) { housing_offers }
          end
        end
      end
    end

    path '/api/dmo/public_housing_offers/{id}' do
      get 'Retrieves a single housing_offer by id' do
        tags TAGS_API_DMO_PUBLIC_HOUSING_OFFERS
        produces 'application/json'
        parameter name: :id, in: :path, type: :string, required: true
        response '200', 'housing_offer found' do
          let(:housing_offer) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE') }
          let(:id) { housing_offer.id }
          include_context 'with integration test'
          include_context 'valid jsonapi ressource' do
            let(:expected_ressource) { housing_offer }
          end
        end
      end
    end
  end
end
